package afwhs.sample

import jp.hishidama.afwhs._

object Sample2Score {
  def run(in: Source[TenModel]) = {
    val split = in.cogroup("split", "", Result[KamokuModel]("out")) {
      (s, r) =>
        s.foreach { ten =>
          r.add(createKamoku(ten, 0, ten.suugaku))
          r.add(createKamoku(ten, 1, ten.kokugo))
          r.add(createKamoku(ten, 2, ten.rika))
          r.add(createKamoku(ten, 3, ten.shakai))
          r.add(createKamoku(ten, 4, ten.eigo))
          r.add(createKamoku(ten, 5, ten.suugaku + ten.kokugo + ten.rika + ten.shakai + ten.eigo))
        }
    }
    println(split)
    val total = split.summarize[KamokuTotal]("total", SAny("kid"), SSum("ten"), SCount("sid" -> "count"))
    println(total)
    val (average, _) = total.convert("average") {
      s =>
        AverageModel(s.kid, s.ten, s.count, s.ten.toDouble / s.count)
    }
    println(average)
    val sigma = split.cogroup("sigma", "kid", "kid" -> average, Result[SigmaModel]("out")) {
      (list, aveList, r) =>
        val am = aveList.head
        val sum = list.map(km => math.pow(km.ten - am.average, 2)).sum
        r.add(SigmaModel(am.kid, am.average, math.sqrt(sum / am.count)))
    }
    println(sigma)
    val score = split.cogroup("score", "kid", "kid" -> sigma, Result[ScoreModel]("out")) {
      (list, sigmaList, r) =>
        val sm = sigmaList.head
        list.foreach { km =>
          r.add(ScoreModel(
            km.sid, km.sname,
            Ȗږ(km.kid),
            km.ten,
            sm.average,
            (km.ten - sm.average) * 10 / sm.sigma + 50))
        }
    }
    score
  }

  def createKamoku(tm: TenModel, kid: Int, ten: Int) =
    KamokuModel(tm.sid, tm.sname, kid, ten)

  val Ȗږ = Array("w", "", "", "Љ", "p", "v")

  def main(args: Array[String]) {
    if (false) {
      val in = Source(
        TenModel("101", "V", 100, 100, 100, 100, 100),
        TenModel("201", "C", 5, 30, 10, 20, 15),
        TenModel("102", "Ђ", 90, 85, 80, 50, 10),
        TenModel("202", "hoge", 10, 30, 25, 45, 20),
        TenModel("103", "foo", 60, 60, 60, 60, 25),
        TenModel("204", "xxx", 80, 77, 90, 40, 44),
        TenModel("205", "yyy", 65, 90, 55, 80, 65),
        TenModel("104", "zzz", 40, 60, 60, 70, 20))
      val r = run(in)
      println(r)
    } else {
      val gen = new Generator("C:/workspace", "afws-wordcount", "afws.wordcount", "Score")
      val r = run(gen.newSource())
      gen.out(r)
      println(gen)
      gen.generateDmdl("afwhs_score.dmdl")
      gen.generateOperator()
      gen.generateImporter()
      gen.generateExporter()
      gen.generateFlow()
    }
  }
}