package jp.hishidama.afwhs

import jp.hishidama.afwhs.prove.CoGroupNode

protected[afwhs] trait CoGroup[S0 <: Model] {
  s0: Source[S0] =>

  def cogroup[R0 <: Model](name:String,key0: Key, r0: Result[R0])(f: (Seq[S0], Result[R0]) => Unit): Source[R0] = {
    this match {
      case g: WithGenerator if g.gen ne null =>
        g.gen.addNode(new CoGroupNode(name, Seq(s0), Seq(key0), Seq(r0), f))
      case _ =>
        if (key0.isEmpty) {
          f(s0.seq, r0)
        } else {
          s0.seq.groupBy(key0.keyGetter(c0)).values.foreach { s => f(s, r0) }
        }
    }
    r0
  }

  def cogroup[S1 <: Model, R0 <: Model](name:String,key0: Key, t1: (Key, Source[S1]), r0: Result[R0])(f: (Seq[S0], Seq[S1], Result[R0]) => Unit)(implicit c1: ClassManifest[S1]): Source[R0] = {
    val (key1, s1) = t1
    checkKeyType(key0 -> c0, key1 -> c1)

    this match {
      case g: WithGenerator if g.gen ne null =>
        g.gen.addNode(new CoGroupNode(name, Seq(s0, s1), Seq(key0, key1), Seq(r0), f))
      case _ =>
        if (key0.isEmpty) {
          f(s0.seq, s1.seq, r0)
        } else {
          val m0 = s0.seq.groupBy(key0.keyGetter(c0))
          val m1 = s1.seq.groupBy(key1.keyGetter(c1))
          val ks = Set.empty ++ m0.keys ++ m1.keys
          ks.foreach { k => f(m0.getOrElse(k, Seq.empty), m1.getOrElse(k, Seq.empty), r0) }
        }
    }

    r0
  }

}
