package jp.hishidama.afwhs

import jp.hishidama.afwhs.prove.SummarizeNode

protected[afwhs] trait Summarize[S0 <: Model] extends AnyRef with Util {
  s0: Source[S0] =>

  //type SAny = SummarizeTypeImpl#SAny

  def summarize[R <: Model](name: String, ss: SummarizeType*)(implicit cm: ClassManifest[R]): Source[R] = {
    ss.foreach { _.setType(c0, cm) }

    val r = Result[R]("out")

    this match {
      case g: WithGenerator if g.gen ne null =>
        g.gen.addNode(new SummarizeNode(name, ss, s0, r))
      case _ =>
        val key = new Key(ss.collect { case s: SAny => s.name }: _*)

        seq.groupBy(key.keyGetter(c0)).values.foreach { seq =>
          val t = createInstance(cm)
          seq.foreach { f =>
            ss.foreach(_.apply(f, t))
          }
          r.add(t)
        }
    }
    r
  }
}

abstract class SummarizeType(val name: String, val to: String) {
  val desc: String
  def apply[A <: Model, B <: Model](f: A, t: B): Unit

  protected var ftype: ClassManifest[_] = _
  protected var ttype: ClassManifest[_] = _
  def setType[A <: Model, B <: Model](ftype: ClassManifest[A], ttype: ClassManifest[B]) {
    this.ftype = ftype
    this.ttype = ttype
  }

  protected lazy val fgetter = {
    ftype.erasure.getMethod(name)
  }
  protected lazy val tgetter = {
    ttype.erasure.getMethod(to)
  }
  protected lazy val tsetter = {
    ttype.erasure.getMethod(to + "_$eq", tgetter.getReturnType())
    //ttype.erasure.getMethods().find(_.getName() == to + "_$eq").get
  }

  override def toString(): String = {
    desc + "(" + name + "->" + to + ")"
  }
}

trait SummarizeTypeImpl {

  class SAny(name: String, to: String) extends SummarizeType(name, to) {
    val desc = "Any"
    def apply[A <: Model, B <: Model](f: A, t: B): Unit = {
      tsetter.invoke(t, fgetter.invoke(f))
    }
  }
  object SAny {
    def apply(name: String, to: String = null) = new SAny(name, Option(to).getOrElse(name))
    def apply(t: (String, String)) = new SAny(t._1, t._2)
  }

  class SSum(name: String, to: String) extends SummarizeType(name, to) {
    val desc = "Sum"
    def apply[A <: Model, B <: Model](f: A, t: B): Unit = {
      val n: Long = tgetter.invoke(t).asInstanceOf[java.lang.Long].longValue()
      val m: Long = fgetter.invoke(f).asInstanceOf[java.lang.Number].longValue()
      tsetter.invoke(t, (n + m).asInstanceOf[AnyRef])
    }
  }
  object SSum {
    def apply(name: String, to: String = null) = new SSum(name, Option(to).getOrElse(name))
    def apply(t: (String, String)) = new SSum(t._1, t._2)
  }

  class SCount(name: String, to: String) extends SummarizeType(name, to) {
    val desc = "Count"
    def apply[A <: Model, B <: Model](f: A, t: B): Unit = {
      val n: Long = tgetter.invoke(t).asInstanceOf[java.lang.Long].longValue()
      tsetter.invoke(t, (n + 1).asInstanceOf[AnyRef])
    }
  }
  object SCount {
    def apply(name: String, to: String = null) = new SCount(name, Option(to).getOrElse(name))
    def apply(t: (String, String)) = new SCount(t._1, t._2)
  }
}
