package jp.hishidama.afwhs

private[afwhs] trait Util {

  def createInstance[A](cm: ClassManifest[A]): A = {
    val c = cm.erasure.getConstructors().head
    val args = c.getParameterTypes().map {
      case c if c == classOf[Int] => java.lang.Integer.valueOf(0)
      case c if c == classOf[Long] => java.lang.Long.valueOf(0)
      case c if c == classOf[Double] => java.lang.Double.valueOf(0)
      case _ => null
    }
    c.newInstance(args.toSeq: _*).asInstanceOf[A]
  }

  def getProperties(cm: ClassManifest[_]): Seq[java.lang.reflect.Method] = {
    cm.erasure.getDeclaredFields().map(f => cm.erasure.getMethod(f.getName())).
      filter(m => try {
        cm.erasure.getMethod(m.getName() + "_$eq", m.getReturnType()); true
      } catch {
        case _ => false
      })
  }

  def dmdlType(c: Class[_]): String = {
    Util.typeMap.apply(c)
  }
}

private object Util {
  val typeMap = Map[Class[_], String](
    classOf[Int] -> "INT",
    classOf[Long] -> "LONG",
    classOf[Double] -> "DOUBLE",
    classOf[String] -> "TEXT")
}