package jp.hishidama.afwhs.gen

import jp.hishidama.afwhs.Generator
import java.io.BufferedWriter

class FlowGen(val gen: Generator) extends Gen {

  var imports = Seq(
    gen.basePackage + ".modelgen.dmdl.model.*",
    gen.gatewayPackage + ".*",
    gen.operatorFactoryPackage + ".*",
    "com.asakusafw.vocabulary.flow.*")

  def generate(w: BufferedWriter) {
    gen.nodes.foreach { _.prepareGenerateFlow(this) }

    w.write("package " + gen.flowPackage + ";\n\n")
    writeImport(w)
    w.write("\n")

    w.write("@JobFlow(name = \"" + gen.flowClass + "\")\n")
    w.write("public class " + gen.flowClass + " extends FlowDescription {\n")

    generateField(w)
    w.write("\n")
    generateConstructor(w)
    w.write("\n")
    generateDescribe(w)

    w.write("}\n")
  }

  private def generateField(w: BufferedWriter) {
    gen.ins.zipWithIndex.foreach {
      case (s, i) =>
        s.flowName = "in" + (if (i == 0) "" else i)
        s.resultName = "this." + s.flowName
    }
    gen.outs.zipWithIndex.foreach {
      case (s, i) =>
        s.flowName = "out" + (if (i == 0) "" else i)
    }

    gen.ins.foreach { s =>
      w.write("\tIn<%s> %s;\n".format(s.modelClassName, s.flowName))
    }
    gen.outs.foreach { s =>
      w.write("\tOut<%s> %s;\n".format(s.modelClassName, s.flowName))
    }
  }

  private def generateConstructor(w: BufferedWriter) {
    w.write("\tpublic " + gen.flowClass + "(\n")
    gen.ins.foreach { s =>
      w.write("\t\t\t@Import(name = \"%s\", description = %sImporter.class) In<%s> %s,\n".format(
        s.flowName, s.modelClassName, s.modelClassName, s.flowName))
    }
    w.write(gen.outs.map { s =>
      "\t\t\t@Export(name = \"%s\", description = %sExporter.class) Out<%s> %s".format(
        s.flowName, s.modelClassName, s.modelClassName, s.flowName)
    }.mkString(",\n"))
    w.write(") {\n")
    gen.ins.foreach { s =>
      w.write("\t\tthis.%s = %1$s;\n".format(s.flowName))
    }
    gen.outs.foreach { s =>
      w.write("\t\tthis.%s = %1$s;\n".format(s.flowName))
    }
    w.write("\t}\n")
  }

  val operator = "operator"

  private def generateDescribe(w: BufferedWriter) {
    w.write("\t%s %s = new %1$s();\n\n".format(gen.operatorFactoryClass, operator))

    w.write("\t@Override\n")
    w.write("\tpublic void describe() {\n")

    gen.nodes.foreach { _.generateFlow(w, this) }

    gen.outs.foreach { s =>
      w.write("\t\tthis.%s.add(%s);\n".format(s.flowName, s.resultName))
    }

    w.write("\t}\n")
  }
}
