package jp.hishidama.afwhs.prove

import jp.hishidama.afwhs._
import java.io.BufferedWriter
import jp.hishidama.afwhs.gen.OperatorGen
import jp.hishidama.afwhs.gen.FlowGen

class CoGroupNode(
  name: String,
  ss: Seq[Source[_ <: Model]],
  val ks: Seq[Key],
  rs: Seq[Result[_ <: Model]],
  f: AnyRef)
  extends Node(name, ss, rs, f) {

  override def toString(): String = {
    "CoGroupNode(" + name + ", ss=" + ss + ", ks=" + ks + ", rs=" + rs + ")"
  }

  override def prepareGenerateOperator(gen: OperatorGen) {
    gen.imports ++= Seq(
      "com.asakusafw.vocabulary.model.Key",
      "java.util.List",
      "com.asakusafw.runtime.core.Result",
      "jp.hishidama.afwhs.java.CoGroupScalaFunction")
  }

  def generateOperator(w: BufferedWriter, gen: OperatorGen) {
    val fname = name + "$function"

    w.append("\n\t@CoGroup\n")
    w.append("\tpublic void " + name + "(\n")
    (ks zip ss zipWithIndex).foreach {
      case ((k, s), i) =>
        w.append("\t\t\t%s List<%s> s%d,\n".format(k.generate, s.modelClassName, i))
    }
    w.append(rs.map { r =>
      "\t\t\tResult<%s> %s".format(r.modelClassName, r.name)
    }.mkString(",\n"))
    w.append(") {\n")

    w.append("\t\t" + fname + ".apply(")
    for (i <- 0 until ss.size) w.append("s" + i + ", ")
    w.append(rs.map(r => r.name).mkString(", "))
    w.append(");\n")

    w.append("\t}\n")

    w.append("\tprivate CoGroupScalaFunction %s = new CoGroupScalaFunction(\"%s\", %d, %d);\n".format(
      fname, f.getClass().getName(), ss.size, rs.size))
  }
}
