package jp.hishidama.afwhs.prove

import jp.hishidama.afwhs._
import java.io.BufferedWriter
import jp.hishidama.afwhs.gen.OperatorGen
import jp.hishidama.afwhs.gen.FlowGen

abstract class Node(
  val name: String,
  val ss: Seq[Source[_ <: Model]],
  val rs: Seq[Result[_ <: Model]],
  val f: AnyRef) {

  rs.foreach { s => s.resultName = name + "." + s.name }

  def prepareGenerateOperator(gen: OperatorGen) {}
  def generateOperator(writer: BufferedWriter, gen: OperatorGen): Unit
  def prepareGenerateFlow(gen: FlowGen) {}

  def generateFlow(w: BufferedWriter, gen: FlowGen) {
    w.write("\t\t%s.%s %s = %s.%s(".format(
      gen.gen.operatorFactoryClass, toCamelCase(name),
      name,
      gen.operator, name))
    w.write(ss.map(_.resultName).mkString(", "))
    w.write(");\n")
  }

  protected def toCamelCase(s: String): String = {
    s(0).toUpperCase + s.substring(1)
  }
}
